@@ -25,6 +25,8 @@ public:
 
 private:
     void checkPlaylistVideoUrls(const QUrl &url);
+    QString decryptSignature(const QString &s);
+    QString reverseString(const QString &string);
 
 private slots:
     void checkUrlIsValid();
